# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Lists AlloyDB endpoints."""


from googlecloudsdk.api_lib.alloydb import api_util
from googlecloudsdk.calliope import base
from googlecloudsdk.core import properties

ENDPOINT_FORMAT = """
    table(
        name,
        endpointType:label=TYPE,
        state:label=STATUS
    )
"""


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class List(base.ListCommand):
  """List AlloyDB endpoints in a given project and region.

  List AlloyDB endpoints in a given project in alphabetical
  order based on the endpoint name.
  """

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
        To list endpoints, run:

          $ {command} --region=us-central1
        """,
  }

  @staticmethod
  def Args(parser):
    """Constructs and sends request.

    Args:
      parser: argparse.Parser: Parser object for command line inputs.
    """
    parser.add_argument(
        '--region',
        default='-',
        help=('Regional location (e.g. asia-east1, us-east1). See the full '
              'list of regions at '
              'https://cloud.google.com/sql/docs/instance-locations. '
              'Default: list endpoints in all regions.'))
    parser.display_info.AddFormat(ENDPOINT_FORMAT)

  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
          arguments specified in the .Args() method.

    Returns:
      ProcessHttpResponse of the request made.
    """
    client = api_util.AlloyDBClient(self.ReleaseTrack())
    alloydb_client = client.alloydb_client
    alloydb_messages = client.alloydb_messages
    location_ref = client.resource_parser.Create(
        'alloydb.projects.locations',
        projectsId=properties.VALUES.core.project.GetOrFail,
        locationsId=args.region)

    result = api_util.YieldFromListHandlingUnreachable(
        alloydb_client.projects_locations_endpoints,
        alloydb_messages.AlloydbProjectsLocationsEndpointsListRequest(
            parent=location_ref.RelativeName()
        ),
        field='endpoints',
        limit=args.limit,
        batch_size=args.page_size,
        batch_size_attribute='pageSize',
    )

    return result



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Compute configuration utilities for clusters command group."""

from __future__ import annotations

from collections.abc import Mapping, MutableMapping, MutableSet, Sequence
from typing import Any

from googlecloudsdk.command_lib.cluster_director.clusters import _validator
from googlecloudsdk.command_lib.cluster_director.clusters import errors

ClusterDirectorError = errors.ClusterDirectorError

# Error messages
_COMPUTE_INSTANCE_ALREADY_EXISTS_ERROR = (
    "Compute instances with id={id} already exist."
)
_COMPUTE_INSTANCE_NOT_FOUND_ERROR = "Compute instances with id={id} not found."


def _GetReservationZone(reservation: str) -> str:
  """Returns the reservation zone from a reservation resource string.

  Args:
    reservation: The reservation resource string.

  Returns:
    The zone extracted from the reservation string.

  Raises:
    ClusterDirectorError: If the reservation string does not contain a zone.
  """

  # projects/{project}/zones/{zone}/reservations/{reservation}/reservationBlocks/{reservationBlock}
  parts = reservation.split("/")
  for current_part, next_part in zip(parts, parts[1:]):
    if current_part == "zones" and next_part:
      return next_part
  raise ClusterDirectorError(
      f"Reservation {reservation} does not contain a zone."
  )


def _GetReservationName(cluster_ref: Any, reservation: str) -> str:
  """Returns the full reservation name including project.

  Args:
    cluster_ref: The cluster resource reference.
    reservation: The reservation string, which may or may not include
      'projects/'.

  Returns:
    The full reservation name.

  Raises:
    ClusterDirectorError: If the reservation string does not contain a zone.
  """
  project = cluster_ref.Parent().projectsId
  if reservation.startswith("projects/"):
    reservation_name = reservation
  else:
    reservation_name = f"projects/{project}/{reservation}"
  _GetReservationZone(reservation)
  return reservation_name


def _MakeOnDemandComputeResource(
    message_module: Any, instance: Mapping[str, Any]
) -> Any:
  """Makes a cluster compute resource message for on demand instances.

  Args:
    message_module: The API message module.
    instance: A dictionary containing on-demand instance configuration.

  Returns:
    A message_module.ComputeResource object for on-demand instances.
  """
  return message_module.ComputeResource(
      config=message_module.ComputeResourceConfig(
          newOnDemandInstances=message_module.NewOnDemandInstancesConfig(
              zone=instance.get("zone"),
              machineType=instance.get("machineType"),
          ),
      ),
  )


def _MakeSpotComputeResource(
    message_module: Any, instance: Mapping[str, Any]
) -> Any:
  """Makes a cluster compute resource message for spot instances.

  Args:
    message_module: The API message module.
    instance: A dictionary containing spot instance configuration.

  Returns:
    A message_module.ComputeResource object for spot instances.
  """
  return message_module.ComputeResource(
      config=message_module.ComputeResourceConfig(
          newSpotInstances=message_module.NewSpotInstancesConfig(
              zone=instance.get("zone"),
              machineType=instance.get("machineType"),
              terminationAction=instance.get("terminationAction"),
          ),
      ),
  )


def _MakeReservedComputeResource(
    message_module: Any, cluster_ref: Any, instance: Mapping[str, Any]
) -> Any:
  """Makes a cluster compute resource message for reserved instances.

  Args:
    message_module: The API message module.
    cluster_ref: The cluster resource reference.
    instance: A dictionary containing reserved instance configuration.

  Returns:
    A message_module.ComputeResource object for reserved instances.

  Raises:
    ClusterDirectorError: If not exactly one of reservation, reservationBlock,
      or reservationSubBlock is provided.
  """
  reservation = instance.get("reservation")
  reservation_block = instance.get("reservationBlock")
  reservation_sub_block = instance.get("reservationSubBlock")
  if (
      sum(
          p is not None
          for p in [reservation, reservation_block, reservation_sub_block]
      )
      != 1
  ):
    raise ClusterDirectorError(
        "Exactly one of reservation, reservationBlock, or"
        " reservationSubBlock must be provided for reserved instances."
    )
  if reservation:
    return message_module.ComputeResource(
        config=message_module.ComputeResourceConfig(
            newReservedInstances=message_module.NewReservedInstancesConfig(
                reservation=_GetReservationName(cluster_ref, reservation),
            ),
        ),
    )
  elif reservation_block:
    return message_module.ComputeResource(
        config=message_module.ComputeResourceConfig(
            newReservedInstances=message_module.NewReservedInstancesConfig(
                reservationBlock=_GetReservationName(
                    cluster_ref, reservation_block
                ),
            ),
        ),
    )
  else:
    return message_module.ComputeResource(
        config=message_module.ComputeResourceConfig(
            newReservedInstances=message_module.NewReservedInstancesConfig(
                reservationSubBlock=_GetReservationName(
                    cluster_ref, reservation_sub_block
                ),
            ),
        ),
    )


def _MakeFlexStartComputeResource(
    message_module: Any, instance: Mapping[str, Any]
) -> Any:
  """Makes a cluster compute resource message for flex start instances.

  Args:
    message_module: The API message module.
    instance: A dictionary containing flex start instance configuration.

  Returns:
    A message_module.ComputeResource object for flex start instances.
  """
  return message_module.ComputeResource(
      config=message_module.ComputeResourceConfig(
          newFlexStartInstances=message_module.NewFlexStartInstancesConfig(
              zone=instance.get("zone"),
              machineType=instance.get("machineType"),
              maxDuration=instance.get("maxDuration"),
          ),
      ),
  )


def MakeClusterCompute(args: Any, message_module: Any, cluster_ref: Any) -> Any:
  """Makes a cluster message with compute fields.

  Args:
    args: The argparse namespace.
    message_module: The API message module.
    cluster_ref: The cluster resource reference.

  Returns:
    A message_module.Cluster.ComputeResourcesValue object.

  Raises:
    ClusterDirectorError: If no compute instances are specified, or if
      duplicate compute instance ids are provided.
  """
  if (
      not args.IsSpecified("on_demand_instances")
      and not args.IsSpecified("spot_instances")
      and not args.IsSpecified("reserved_instances")
      and not args.IsSpecified("flex_start_instances")
  ):
    raise ClusterDirectorError(
        "At least one of on_demand_instances, spot_instances,"
        " reserved_instances, or flex_start_instances flag must be specified."
    )
  compute_ids = set()
  compute = message_module.Cluster.ComputeResourcesValue()
  if args.IsSpecified("on_demand_instances"):
    for instance in args.on_demand_instances:
      compute_id = instance.get("id")
      _validator.ValidateResourceID(compute_id)
      compute_ids.add(compute_id)
      compute.additionalProperties.append(
          message_module.Cluster.ComputeResourcesValue.AdditionalProperty(
              key=compute_id,
              value=_MakeOnDemandComputeResource(message_module, instance),
          )
      )
  if args.IsSpecified("spot_instances"):
    for instance in args.spot_instances:
      compute_id = instance.get("id")
      _validator.ValidateResourceID(compute_id)
      compute_ids.add(compute_id)
      compute.additionalProperties.append(
          message_module.Cluster.ComputeResourcesValue.AdditionalProperty(
              key=compute_id,
              value=_MakeSpotComputeResource(message_module, instance),
          )
      )
  if args.IsSpecified("reserved_instances"):
    for instance in args.reserved_instances:
      compute_id = instance.get("id")
      _validator.ValidateResourceID(compute_id)
      compute_ids.add(compute_id)
      compute.additionalProperties.append(
          message_module.Cluster.ComputeResourcesValue.AdditionalProperty(
              key=compute_id,
              value=_MakeReservedComputeResource(
                  message_module, cluster_ref, instance
              ),
          )
      )
  if args.IsSpecified("flex_start_instances"):
    for instance in args.flex_start_instances:
      compute_id = instance.get("id")
      _validator.ValidateResourceID(compute_id)
      compute_ids.add(compute_id)
      compute.additionalProperties.append(
          message_module.Cluster.ComputeResourcesValue.AdditionalProperty(
              key=compute_id,
              value=_MakeFlexStartComputeResource(message_module, instance),
          )
      )
  if len(compute_ids) != len(compute.additionalProperties):
    raise ClusterDirectorError(
        "Compute instances with duplicate ids are not supported."
    )
  return compute


def _ConvertMessageToDict(message: Any) -> dict[str, Any]:
  """Converts a message with AdditionalProperties to a dict.

  The input message is expected to contain a list of type
  AdditionalProperty(key=str, value=Any).

  Args:
    message: The message to convert.

  Returns:
    A dictionary representation of the message.
  """
  if not message:
    return {}
  return {each.key: each.value for each in message.additionalProperties}


def _AddKeyToDictSpec(
    key: str,
    dict_spec: MutableMapping[str, Any],
    value: Any,
    exception_message: str,
) -> None:
  """Adds a cluster identifier (key) with value, if not present in dict spec.

  Args:
    key: The key to add.
    dict_spec: The dictionary to add to.
    value: The value to add for the key.
    exception_message: The exception message template to use if key exists.

  Raises:
    ClusterDirectorError: If key is already in dict_spec.
  """
  if key in dict_spec:
    raise ClusterDirectorError(exception_message.format(id=key))
  dict_spec[key] = value


def _RemoveKeyByAttrFromDictSpec(
    key: str,
    dict_spec: MutableMapping[str, Any],
    attrs: Sequence[str],
    key_exception_message: str,
    attr_exception_message: str,
) -> None:
  """Removes a cluster identifier (key) by attribute, if present in dict spec.

  Checks if the entry in dict_spec for key has one of the attributes in attrs
  before removing.

  Args:
    key: The key to remove.
    dict_spec: The dictionary to remove from.
    attrs: A list of attribute names to check for existence on
      dict_spec[key].config.
    key_exception_message: Exception message template if key is not in
      dict_spec.
    attr_exception_message: Exception message template if dict_spec[key] does
      not have a 'config' attribute or any of attrs.

  Raises:
    ClusterDirectorError: If key is not in dict_spec, or if none of attrs
      are present for dict_spec[key].config.
  """
  if key not in dict_spec:
    raise ClusterDirectorError(key_exception_message.format(id=key))
  if not getattr(dict_spec[key], "config", None):
    raise ClusterDirectorError(attr_exception_message.format(id=key))
  if not any(getattr(dict_spec[key].config, attr, None) for attr in attrs):
    raise ClusterDirectorError(attr_exception_message.format(id=key))
  dict_spec.pop(key)


def MakeClusterComputePatch(
    args: Any,
    message_module: Any,
    cluster_ref: Any,
    existing_cluster: Any,
    update_mask: MutableSet[str],
) -> Any:
  """Makes a cluster compute patch message with compute fields.

  Args:
    args: The argparse namespace.
    message_module: The API message module.
    cluster_ref: The cluster resource reference.
    existing_cluster: The existing cluster message.
    update_mask: A set of field paths to update.

  Returns:
    A message_module.Cluster.ComputeResourcesValue object with patched fields.

  Raises:
    ClusterDirectorError: If compute instances becomes empty after patch.
  """
  compute_resources = message_module.Cluster.ComputeResourcesValue()
  compute_by_id = _ConvertMessageToDict(
      existing_cluster.computeResources if existing_cluster else None
  )
  is_compute_updated = False
  if args.IsSpecified("remove_on_demand_instances"):
    for compute_id in args.remove_on_demand_instances:
      _RemoveKeyByAttrFromDictSpec(
          key=compute_id,
          dict_spec=compute_by_id,
          attrs=["newOnDemandInstances"],
          key_exception_message=_COMPUTE_INSTANCE_NOT_FOUND_ERROR,
          attr_exception_message=(
              f"On demand {_COMPUTE_INSTANCE_NOT_FOUND_ERROR}"
          ),
      )
      is_compute_updated = True
  if args.IsSpecified("remove_spot_instances"):
    for compute_id in args.remove_spot_instances:
      _RemoveKeyByAttrFromDictSpec(
          key=compute_id,
          dict_spec=compute_by_id,
          attrs=["newSpotInstances"],
          key_exception_message=_COMPUTE_INSTANCE_NOT_FOUND_ERROR,
          attr_exception_message=f"Spot {_COMPUTE_INSTANCE_NOT_FOUND_ERROR}",
      )
      is_compute_updated = True
  if args.IsSpecified("remove_reserved_instances"):
    for compute_id in args.remove_reserved_instances:
      _RemoveKeyByAttrFromDictSpec(
          key=compute_id,
          dict_spec=compute_by_id,
          attrs=["newReservedInstances"],
          key_exception_message=_COMPUTE_INSTANCE_NOT_FOUND_ERROR,
          attr_exception_message=(
              f"Reserved {_COMPUTE_INSTANCE_NOT_FOUND_ERROR}"
          ),
      )
      is_compute_updated = True
  if args.IsSpecified("remove_flex_start_instances"):
    for compute_id in args.remove_flex_start_instances:
      _RemoveKeyByAttrFromDictSpec(
          key=compute_id,
          dict_spec=compute_by_id,
          attrs=["newFlexStartInstances"],
          key_exception_message=_COMPUTE_INSTANCE_NOT_FOUND_ERROR,
          attr_exception_message=(
              f"Flex Start {_COMPUTE_INSTANCE_NOT_FOUND_ERROR}"
          ),
      )
      is_compute_updated = True
  if args.IsSpecified("add_on_demand_instances"):
    for instance in args.add_on_demand_instances:
      compute_id = instance.get("id")
      _validator.ValidateResourceID(compute_id)
      _AddKeyToDictSpec(
          key=compute_id,
          dict_spec=compute_by_id,
          value=_MakeOnDemandComputeResource(message_module, instance),
          exception_message=_COMPUTE_INSTANCE_ALREADY_EXISTS_ERROR,
      )
      is_compute_updated = True
  if args.IsSpecified("add_spot_instances"):
    for instance in args.add_spot_instances:
      compute_id = instance.get("id")
      _validator.ValidateResourceID(compute_id)
      _AddKeyToDictSpec(
          key=compute_id,
          dict_spec=compute_by_id,
          value=_MakeSpotComputeResource(message_module, instance),
          exception_message=_COMPUTE_INSTANCE_ALREADY_EXISTS_ERROR,
      )
      is_compute_updated = True
  if args.IsSpecified("add_reserved_instances"):
    for instance in args.add_reserved_instances:
      compute_id = instance.get("id")
      _validator.ValidateResourceID(compute_id)
      _AddKeyToDictSpec(
          key=compute_id,
          dict_spec=compute_by_id,
          value=_MakeReservedComputeResource(
              message_module, cluster_ref, instance
          ),
          exception_message=_COMPUTE_INSTANCE_ALREADY_EXISTS_ERROR,
      )
      is_compute_updated = True
  if args.IsSpecified("add_flex_start_instances"):
    for instance in args.add_flex_start_instances:
      compute_id = instance.get("id")
      _validator.ValidateResourceID(compute_id)
      _AddKeyToDictSpec(
          key=compute_id,
          dict_spec=compute_by_id,
          value=_MakeFlexStartComputeResource(message_module, instance),
          exception_message=_COMPUTE_INSTANCE_ALREADY_EXISTS_ERROR,
      )
      is_compute_updated = True
  if is_compute_updated:
    compute_resources.additionalProperties = [
        message_module.Cluster.ComputeResourcesValue.AdditionalProperty(
            key=key, value=value
        )
        for key, value in compute_by_id.items()
    ]
    if not compute_resources.additionalProperties:
      raise ClusterDirectorError("Compute instances cannot be empty.")
    update_mask.add("compute.resource_requests")
  return compute_resources



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create a new SSL policy."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.ssl_policies import ssl_policies_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.ssl_policies import flags


@base.UniverseCompatible
class Create(base.CreateCommand):
  """Create a new Compute Engine SSL policy.

  *{command}* creates a new SSL policy.

  An SSL policy specifies the server-side support for SSL features. An SSL
  policy can be attached to a TargetHttpsProxy or a TargetSslProxy. This affects
  connections between clients and the load balancer. SSL
  policies do not affect the connection between the load balancers and the
  backends. SSL policies are used by Application Load Balancers and proxy
  Network Load Balancers.
  """

  SSL_POLICY_ARG = flags.GetSslPolicyMultiScopeArgument()

  @classmethod
  def Args(cls, parser):
    """Set up arguments for this command."""
    parser.display_info.AddFormat(flags.DEFAULT_LIST_FORMAT)
    parser.display_info.AddCacheUpdater(flags.SslPoliciesCompleter)
    cls.SSL_POLICY_ARG.AddArgument(parser, operation_type='create')
    flags.GetDescriptionFlag().AddToParser(parser)
    flags.GetProfileFlag(default='COMPATIBLE').AddToParser(parser)
    flags.GetMinTlsVersionFlag(default='1.0').AddToParser(parser)
    flags.GetCustomFeaturesFlag().AddToParser(parser)
    flags.GetPostQuantumKeyExchangeFlag().AddToParser(parser)

  def Run(self, args):
    """Issues the request to create a new SSL policy."""
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    helper = ssl_policies_utils.SslPolicyHelper(holder)
    ssl_policy_ref = self.SSL_POLICY_ARG.ResolveAsResource(
        args, holder.resources, default_scope=compute_scope.ScopeEnum.GLOBAL)
    custom_features = args.custom_features if args.IsSpecified(
        'custom_features') else []

    ssl_policy_to_insert = helper.GetSslPolicyForInsert(
        name=ssl_policy_ref.Name(),
        description=args.description,
        profile=args.profile,
        min_tls_version=flags.ParseTlsVersion(args.min_tls_version),
        custom_features=custom_features,
        post_quantum_key_exchange=args.post_quantum_key_exchange,
    )
    operation_ref = helper.Create(ssl_policy_ref, ssl_policy_to_insert)
    return helper.WaitForOperation(ssl_policy_ref, operation_ref,
                                   'Creating SSL policy')



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud database migration private connections API."""


from typing import Optional

from googlecloudsdk.api_lib.database_migration import api_util
from googlecloudsdk.core import resources


def GetPrivateConnectionURI(resource):
  private_connection = resources.REGISTRY.ParseRelativeName(
      resource.name,
      collection='datamigration.projects.locations.privateConnections')
  return private_connection.SelfLink()


class PrivateConnectionsClient:
  """Client for private connections service in the API."""

  def __init__(
      self,
      release_track,
      client=None,
      messages=None,
      location: Optional[str] = None,
  ):
    self._client = client or api_util.GetClientInstance(
        release_track,
        location=location,
    )
    self._messages = messages or api_util.GetMessagesModule(release_track)
    self._service = self._client.projects_locations_privateConnections
    self._resource_parser = api_util.GetResourceParser(release_track)

  def _GetPrivateConnection(self, private_connection_id, args):
    """Returns a private connection object."""
    private_connection_obj = self._messages.PrivateConnection(
        name=private_connection_id, labels={}, displayName=args.display_name)
    if args.IsKnownAndSpecified('network_attachment'):
      private_connection_obj.pscInterfaceConfig = (
          self._messages.PscInterfaceConfig(
              networkAttachment=args.network_attachment
          )
      )
    else:
      vpc_peering_ref = args.CONCEPTS.vpc.Parse()
      private_connection_obj.vpcPeeringConfig = self._messages.VpcPeeringConfig(
          vpcName=vpc_peering_ref.RelativeName(), subnet=args.subnet
      )

    return private_connection_obj

  def Create(self, parent_ref, private_connection_id, args=None):
    """Creates a private connection.

    Args:
      parent_ref: a Resource reference to a parent
        datamigration.projects.locations resource for this private connection.
      private_connection_id: str, the name of the resource to create.
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      Operation: the operation for creating the private connection.
    """

    private_connection = self._GetPrivateConnection(private_connection_id, args)

    request_id = api_util.GenerateRequestId()
    create_req_type = (
        self._messages.DatamigrationProjectsLocationsPrivateConnectionsCreateRequest
    )
    create_req = create_req_type(
        privateConnection=private_connection,
        privateConnectionId=private_connection.name,
        parent=parent_ref,
        requestId=request_id,
        validateOnly=args.validate_only)
    if args.IsKnownAndSpecified('skip_validation'):
      create_req.skipValidation = True

    return self._service.Create(create_req)

  def Delete(self, private_connection_name):
    """Deletes a private connection.

    Args:
      private_connection_name: str, the name of the resource to delete.

    Returns:
      Operation: the operation for deleting the private connection.
    """

    request_id = api_util.GenerateRequestId()
    delete_req_type = (
        self._messages
        .DatamigrationProjectsLocationsPrivateConnectionsDeleteRequest)
    delete_req = delete_req_type(
        name=private_connection_name, requestId=request_id)

    return self._service.Delete(delete_req)



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update command to update a new Custom Intercept profile."""

from googlecloudsdk.api_lib.network_security.security_profiles import intercept_api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.network_security import sp_flags
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log

DETAILED_HELP = {
    'DESCRIPTION': """

          Update a Custom Intercept Security Profile.

          The supported fields for update are `description` and `labels`.

        """,
    'EXAMPLES': """
          To update the description of a Custom Intercept Security Profile named `intercept-profile`, run:

              $ {command} intercept-profile --description="A new description" \
              --organization=1234567890 --location=global

          To change the labels of a Custom Intercept Security Profile named `intercept-profile`, run:

              $ {command} intercept-profile
              --update-labels=key1=value1,key2=value2  \
              --delete-labels=key3,key4 \
              --organization=1234567890 --location=glob
        """,
}

_PROJECT_SCOPE_SUPPORTED_TRACKS = (
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
)


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Update(base.UpdateCommand):
  """Updates a Custom Intercept Profile."""

  detailed_help = DETAILED_HELP

  @classmethod
  def Args(cls, parser):
    project_scope_supported = (
        cls.ReleaseTrack() in _PROJECT_SCOPE_SUPPORTED_TRACKS
    )
    sp_flags.AddSecurityProfileResource(
        parser, cls.ReleaseTrack(), project_scope_supported
    )
    sp_flags.AddProfileDescription(parser)
    base.ASYNC_FLAG.AddToParser(parser)
    base.ASYNC_FLAG.SetDefault(parser, False)
    labels_util.AddUpdateLabelsFlags(parser)

  def Run(self, args):
    result = args.CONCEPTS.security_profile.Parse()
    security_profile = result.result
    description = args.description
    is_async = args.async_

    project_scoped = (
        result.concept_type.name
        == sp_flags.PROJECT_SECURITY_PROFILE_RESOURCE_COLLECTION
    )
    client = intercept_api.Client(self.ReleaseTrack(), project_scoped)

    labels_update = labels_util.ProcessUpdateArgsLazy(
        args,
        client.messages.SecurityProfile.LabelsValue,
        orig_labels_thunk=lambda: self.getLabels(client, security_profile),
    )

    response = client.UpdateSecurityProfile(
        name=security_profile.RelativeName(),
        description=description,
        labels=labels_update.GetOrNone(),
    )

    # Return the in-progress operation if async is requested.
    if is_async:
      operation_id = response.name
      log.status.Print(
          'Check for operation completion status using operation ID:',
          operation_id,
      )
      return response

    # Default operation poller if async is not specified.
    return client.WaitForOperation(
        operation_ref=client.GetOperationsRef(response),
        message='Waiting for security-profile [{}] to be updated'.format(
            security_profile.RelativeName()
        ),
        has_result=True,
    )

  def getLabels(self, client, security_profile):
    return client.GetSecurityProfile(security_profile.RelativeName()).labels



"""Generated client library for secretmanager version v1beta2."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.secretmanager.v1beta2 import secretmanager_v1beta2_messages as messages


class SecretmanagerV1beta2(base_api.BaseApiClient):
  """Generated client library for service secretmanager version v1beta2."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://secretmanager.googleapis.com/'
  MTLS_BASE_URL = 'https://secretmanager.mtls.googleapis.com/'

  _PACKAGE = 'secretmanager'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1beta2'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'SecretmanagerV1beta2'
  _URL_VERSION = 'v1beta2'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new secretmanager handle."""
    url = url or self.BASE_URL
    super(SecretmanagerV1beta2, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_secrets_versions = self.ProjectsLocationsSecretsVersionsService(self)
    self.projects_locations_secrets = self.ProjectsLocationsSecretsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects_secrets_versions = self.ProjectsSecretsVersionsService(self)
    self.projects_secrets = self.ProjectsSecretsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsSecretsVersionsService(base_api.BaseApiService):
    """Service class for the projects_locations_secrets_versions resource."""

    _NAME = 'projects_locations_secrets_versions'

    def __init__(self, client):
      super(SecretmanagerV1beta2.ProjectsLocationsSecretsVersionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Access(self, request, global_params=None):
      r"""Accesses a SecretVersion. This call returns the secret data. `projects/*/secrets/*/versions/latest` is an alias to the most recently created SecretVersion.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsVersionsAccessRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AccessSecretVersionResponse) The response message.
      """
      config = self.GetMethodConfig('Access')
      return self._RunMethod(
          config, request, global_params=global_params)

    Access.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}/versions/{versionsId}:access',
        http_method='GET',
        method_id='secretmanager.projects.locations.secrets.versions.access',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:access',
        request_field='',
        request_type_name='SecretmanagerProjectsLocationsSecretsVersionsAccessRequest',
        response_type_name='AccessSecretVersionResponse',
        supports_download=False,
    )

    def Destroy(self, request, global_params=None):
      r"""Destroys a SecretVersion. Sets the state of the SecretVersion to DESTROYED and irrevocably destroys the secret data.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsVersionsDestroyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecretVersion) The response message.
      """
      config = self.GetMethodConfig('Destroy')
      return self._RunMethod(
          config, request, global_params=global_params)

    Destroy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}/versions/{versionsId}:destroy',
        http_method='POST',
        method_id='secretmanager.projects.locations.secrets.versions.destroy',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:destroy',
        request_field='destroySecretVersionRequest',
        request_type_name='SecretmanagerProjectsLocationsSecretsVersionsDestroyRequest',
        response_type_name='SecretVersion',
        supports_download=False,
    )

    def Disable(self, request, global_params=None):
      r"""Disables a SecretVersion. Sets the state of the SecretVersion to DISABLED.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsVersionsDisableRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecretVersion) The response message.
      """
      config = self.GetMethodConfig('Disable')
      return self._RunMethod(
          config, request, global_params=global_params)

    Disable.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}/versions/{versionsId}:disable',
        http_method='POST',
        method_id='secretmanager.projects.locations.secrets.versions.disable',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:disable',
        request_field='disableSecretVersionRequest',
        request_type_name='SecretmanagerProjectsLocationsSecretsVersionsDisableRequest',
        response_type_name='SecretVersion',
        supports_download=False,
    )

    def Enable(self, request, global_params=None):
      r"""Enables a SecretVersion. Sets the state of the SecretVersion to ENABLED.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsVersionsEnableRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecretVersion) The response message.
      """
      config = self.GetMethodConfig('Enable')
      return self._RunMethod(
          config, request, global_params=global_params)

    Enable.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}/versions/{versionsId}:enable',
        http_method='POST',
        method_id='secretmanager.projects.locations.secrets.versions.enable',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:enable',
        request_field='enableSecretVersionRequest',
        request_type_name='SecretmanagerProjectsLocationsSecretsVersionsEnableRequest',
        response_type_name='SecretVersion',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets metadata for a SecretVersion. `projects/*/secrets/*/versions/latest` is an alias to the most recently created SecretVersion.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsVersionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecretVersion) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}/versions/{versionsId}',
        http_method='GET',
        method_id='secretmanager.projects.locations.secrets.versions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecretmanagerProjectsLocationsSecretsVersionsGetRequest',
        response_type_name='SecretVersion',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists SecretVersions. This call does not return secret data.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsVersionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSecretVersionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}/versions',
        http_method='GET',
        method_id='secretmanager.projects.locations.secrets.versions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1beta2/{+parent}/versions',
        request_field='',
        request_type_name='SecretmanagerProjectsLocationsSecretsVersionsListRequest',
        response_type_name='ListSecretVersionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsSecretsService(base_api.BaseApiService):
    """Service class for the projects_locations_secrets resource."""

    _NAME = 'projects_locations_secrets'

    def __init__(self, client):
      super(SecretmanagerV1beta2.ProjectsLocationsSecretsService, self).__init__(client)
      self._upload_configs = {
          }

    def AddVersion(self, request, global_params=None):
      r"""Creates a new SecretVersion containing secret data and attaches it to an existing Secret.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsAddVersionRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecretVersion) The response message.
      """
      config = self.GetMethodConfig('AddVersion')
      return self._RunMethod(
          config, request, global_params=global_params)

    AddVersion.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}:addVersion',
        http_method='POST',
        method_id='secretmanager.projects.locations.secrets.addVersion',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1beta2/{+parent}:addVersion',
        request_field='addSecretVersionRequest',
        request_type_name='SecretmanagerProjectsLocationsSecretsAddVersionRequest',
        response_type_name='SecretVersion',
        supports_download=False,
    )

    def Create(self, request, global_params=None):
      r"""Creates a new Secret containing no SecretVersions.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Secret) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets',
        http_method='POST',
        method_id='secretmanager.projects.locations.secrets.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['secretId'],
        relative_path='v1beta2/{+parent}/secrets',
        request_field='secret',
        request_type_name='SecretmanagerProjectsLocationsSecretsCreateRequest',
        response_type_name='Secret',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a Secret.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}',
        http_method='DELETE',
        method_id='secretmanager.projects.locations.secrets.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['etag'],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecretmanagerProjectsLocationsSecretsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets metadata for a given Secret.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Secret) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}',
        http_method='GET',
        method_id='secretmanager.projects.locations.secrets.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecretmanagerProjectsLocationsSecretsGetRequest',
        response_type_name='Secret',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a secret. Returns empty policy if the secret exists and does not have a policy set.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}:getIamPolicy',
        http_method='GET',
        method_id='secretmanager.projects.locations.secrets.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1beta2/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='SecretmanagerProjectsLocationsSecretsGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Secrets.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSecretsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets',
        http_method='GET',
        method_id='secretmanager.projects.locations.secrets.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1beta2/{+parent}/secrets',
        request_field='',
        request_type_name='SecretmanagerProjectsLocationsSecretsListRequest',
        response_type_name='ListSecretsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates metadata of an existing Secret.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Secret) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}',
        http_method='PATCH',
        method_id='secretmanager.projects.locations.secrets.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='secret',
        request_type_name='SecretmanagerProjectsLocationsSecretsPatchRequest',
        response_type_name='Secret',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified secret. Replaces any existing policy. Permissions on SecretVersions are enforced according to the policy set on the associated Secret.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}:setIamPolicy',
        http_method='POST',
        method_id='secretmanager.projects.locations.secrets.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta2/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='SecretmanagerProjectsLocationsSecretsSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has for the specified secret. If the secret does not exist, this call returns an empty set of permissions, not a NOT_FOUND error. Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (SecretmanagerProjectsLocationsSecretsTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/secrets/{secretsId}:testIamPermissions',
        http_method='POST',
        method_id='secretmanager.projects.locations.secrets.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta2/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='SecretmanagerProjectsLocationsSecretsTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(SecretmanagerV1beta2.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (SecretmanagerProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='secretmanager.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecretmanagerProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the ListLocationsRequest.name field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (SecretmanagerProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations',
        http_method='GET',
        method_id='secretmanager.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1beta2/{+name}/locations',
        request_field='',
        request_type_name='SecretmanagerProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsSecretsVersionsService(base_api.BaseApiService):
    """Service class for the projects_secrets_versions resource."""

    _NAME = 'projects_secrets_versions'

    def __init__(self, client):
      super(SecretmanagerV1beta2.ProjectsSecretsVersionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Access(self, request, global_params=None):
      r"""Accesses a SecretVersion. This call returns the secret data. `projects/*/secrets/*/versions/latest` is an alias to the most recently created SecretVersion.

      Args:
        request: (SecretmanagerProjectsSecretsVersionsAccessRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AccessSecretVersionResponse) The response message.
      """
      config = self.GetMethodConfig('Access')
      return self._RunMethod(
          config, request, global_params=global_params)

    Access.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}/versions/{versionsId}:access',
        http_method='GET',
        method_id='secretmanager.projects.secrets.versions.access',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:access',
        request_field='',
        request_type_name='SecretmanagerProjectsSecretsVersionsAccessRequest',
        response_type_name='AccessSecretVersionResponse',
        supports_download=False,
    )

    def Destroy(self, request, global_params=None):
      r"""Destroys a SecretVersion. Sets the state of the SecretVersion to DESTROYED and irrevocably destroys the secret data.

      Args:
        request: (SecretmanagerProjectsSecretsVersionsDestroyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecretVersion) The response message.
      """
      config = self.GetMethodConfig('Destroy')
      return self._RunMethod(
          config, request, global_params=global_params)

    Destroy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}/versions/{versionsId}:destroy',
        http_method='POST',
        method_id='secretmanager.projects.secrets.versions.destroy',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:destroy',
        request_field='destroySecretVersionRequest',
        request_type_name='SecretmanagerProjectsSecretsVersionsDestroyRequest',
        response_type_name='SecretVersion',
        supports_download=False,
    )

    def Disable(self, request, global_params=None):
      r"""Disables a SecretVersion. Sets the state of the SecretVersion to DISABLED.

      Args:
        request: (SecretmanagerProjectsSecretsVersionsDisableRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecretVersion) The response message.
      """
      config = self.GetMethodConfig('Disable')
      return self._RunMethod(
          config, request, global_params=global_params)

    Disable.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}/versions/{versionsId}:disable',
        http_method='POST',
        method_id='secretmanager.projects.secrets.versions.disable',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:disable',
        request_field='disableSecretVersionRequest',
        request_type_name='SecretmanagerProjectsSecretsVersionsDisableRequest',
        response_type_name='SecretVersion',
        supports_download=False,
    )

    def Enable(self, request, global_params=None):
      r"""Enables a SecretVersion. Sets the state of the SecretVersion to ENABLED.

      Args:
        request: (SecretmanagerProjectsSecretsVersionsEnableRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecretVersion) The response message.
      """
      config = self.GetMethodConfig('Enable')
      return self._RunMethod(
          config, request, global_params=global_params)

    Enable.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}/versions/{versionsId}:enable',
        http_method='POST',
        method_id='secretmanager.projects.secrets.versions.enable',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:enable',
        request_field='enableSecretVersionRequest',
        request_type_name='SecretmanagerProjectsSecretsVersionsEnableRequest',
        response_type_name='SecretVersion',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets metadata for a SecretVersion. `projects/*/secrets/*/versions/latest` is an alias to the most recently created SecretVersion.

      Args:
        request: (SecretmanagerProjectsSecretsVersionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecretVersion) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}/versions/{versionsId}',
        http_method='GET',
        method_id='secretmanager.projects.secrets.versions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecretmanagerProjectsSecretsVersionsGetRequest',
        response_type_name='SecretVersion',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists SecretVersions. This call does not return secret data.

      Args:
        request: (SecretmanagerProjectsSecretsVersionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSecretVersionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}/versions',
        http_method='GET',
        method_id='secretmanager.projects.secrets.versions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1beta2/{+parent}/versions',
        request_field='',
        request_type_name='SecretmanagerProjectsSecretsVersionsListRequest',
        response_type_name='ListSecretVersionsResponse',
        supports_download=False,
    )

  class ProjectsSecretsService(base_api.BaseApiService):
    """Service class for the projects_secrets resource."""

    _NAME = 'projects_secrets'

    def __init__(self, client):
      super(SecretmanagerV1beta2.ProjectsSecretsService, self).__init__(client)
      self._upload_configs = {
          }

    def AddVersion(self, request, global_params=None):
      r"""Creates a new SecretVersion containing secret data and attaches it to an existing Secret.

      Args:
        request: (SecretmanagerProjectsSecretsAddVersionRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecretVersion) The response message.
      """
      config = self.GetMethodConfig('AddVersion')
      return self._RunMethod(
          config, request, global_params=global_params)

    AddVersion.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}:addVersion',
        http_method='POST',
        method_id='secretmanager.projects.secrets.addVersion',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1beta2/{+parent}:addVersion',
        request_field='addSecretVersionRequest',
        request_type_name='SecretmanagerProjectsSecretsAddVersionRequest',
        response_type_name='SecretVersion',
        supports_download=False,
    )

    def Create(self, request, global_params=None):
      r"""Creates a new Secret containing no SecretVersions.

      Args:
        request: (SecretmanagerProjectsSecretsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Secret) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets',
        http_method='POST',
        method_id='secretmanager.projects.secrets.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['secretId'],
        relative_path='v1beta2/{+parent}/secrets',
        request_field='secret',
        request_type_name='SecretmanagerProjectsSecretsCreateRequest',
        response_type_name='Secret',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a Secret.

      Args:
        request: (SecretmanagerProjectsSecretsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}',
        http_method='DELETE',
        method_id='secretmanager.projects.secrets.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['etag'],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecretmanagerProjectsSecretsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets metadata for a given Secret.

      Args:
        request: (SecretmanagerProjectsSecretsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Secret) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}',
        http_method='GET',
        method_id='secretmanager.projects.secrets.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecretmanagerProjectsSecretsGetRequest',
        response_type_name='Secret',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a secret. Returns empty policy if the secret exists and does not have a policy set.

      Args:
        request: (SecretmanagerProjectsSecretsGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}:getIamPolicy',
        http_method='GET',
        method_id='secretmanager.projects.secrets.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1beta2/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='SecretmanagerProjectsSecretsGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Secrets.

      Args:
        request: (SecretmanagerProjectsSecretsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSecretsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets',
        http_method='GET',
        method_id='secretmanager.projects.secrets.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1beta2/{+parent}/secrets',
        request_field='',
        request_type_name='SecretmanagerProjectsSecretsListRequest',
        response_type_name='ListSecretsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates metadata of an existing Secret.

      Args:
        request: (SecretmanagerProjectsSecretsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Secret) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}',
        http_method='PATCH',
        method_id='secretmanager.projects.secrets.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='secret',
        request_type_name='SecretmanagerProjectsSecretsPatchRequest',
        response_type_name='Secret',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified secret. Replaces any existing policy. Permissions on SecretVersions are enforced according to the policy set on the associated Secret.

      Args:
        request: (SecretmanagerProjectsSecretsSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}:setIamPolicy',
        http_method='POST',
        method_id='secretmanager.projects.secrets.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta2/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='SecretmanagerProjectsSecretsSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has for the specified secret. If the secret does not exist, this call returns an empty set of permissions, not a NOT_FOUND error. Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (SecretmanagerProjectsSecretsTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/secrets/{secretsId}:testIamPermissions',
        http_method='POST',
        method_id='secretmanager.projects.secrets.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta2/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='SecretmanagerProjectsSecretsTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(SecretmanagerV1beta2.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for adding a path matcher to a URL map."""

import collections

from apitools.base.py import encoding
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import exceptions as compute_exceptions
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.backend_buckets import (
    flags as backend_bucket_flags,
)
from googlecloudsdk.command_lib.compute.backend_services import (
    flags as backend_service_flags,
)
from googlecloudsdk.command_lib.compute.url_maps import flags
from googlecloudsdk.command_lib.compute.url_maps import url_maps_utils
from googlecloudsdk.core import properties


def _DetailedHelp():
  # pylint:disable=line-too-long
  return {
      'brief': 'Add a path matcher to a URL map.',
      'DESCRIPTION': (
          """
*{command}* is used to add a path matcher to a URL map. A path
matcher maps HTTP request paths to backend services or backend
buckets. Each path matcher must be referenced by at least one
host rule. This command can create a new host rule through the
`--new-hosts` flag or it can reconfigure an existing host rule
to point to the newly added path matcher using `--existing-host`.
In the latter case, if a path matcher is orphaned as a result
of the operation, this command will fail unless
`--delete-orphaned-path-matcher` is provided. Path matcher
constraints can be found
[here](https://cloud.google.com/load-balancing/docs/url-map-concepts#pm-constraints).
"""
      ),
      'EXAMPLES': (
          """
To create a rule for mapping the path ```/search/*``` to the
hypothetical ```search-service```, ```/static/*``` to the
```static-bucket``` backend bucket and ```/images/*``` to the
```images-service``` under the hosts ```example.com``` and
```*.example.com```, run:

  $ {command} MY-URL-MAP --path-matcher-name=MY-MATCHER --default-service=MY-DEFAULT-SERVICE --backend-service-path-rules='/search/*=search_service,/images/*=images-service' --backend-bucket-path-rules='/static/*=static-bucket' --new-hosts=example.com '*.example.com'

Note that a default service or default backend bucket must be
provided to handle paths for which there is no mapping.
"""
      ),
  }
  # pylint:enable=line-too-long


def _Args(parser):
  """Common arguments to add-path-matcher commands for each release track."""
  parser.add_argument(
      '--description',
      help='An optional, textual description for the path matcher.',
  )

  parser.add_argument(
      '--path-matcher-name',
      required=True,
      help='The name to assign to the path matcher.',
  )

  parser.add_argument(
      '--path-rules',
      type=arg_parsers.ArgDict(min_length=1),
      default={},
      metavar='PATH=SERVICE',
      help='Rules for mapping request paths to services.',
  )

  host_rule = parser.add_mutually_exclusive_group()
  host_rule.add_argument(
      '--new-hosts',
      type=arg_parsers.ArgList(min_length=1),
      metavar='NEW_HOST',
      help=(
          'If specified, a new host rule with the given hosts is created '
          'and the path matcher is tied to the new host rule.'
      ),
  )

  host_rule.add_argument(
      '--existing-host',
      help="""\
      An existing host rule to tie the new path matcher to. Although
      host rules can contain more than one host, only a single host
      is needed to uniquely identify the host rule.
      """,
  )

  parser.add_argument(
      '--delete-orphaned-path-matcher',
      action='store_true',
      default=False,
      help=(
          'If provided and a path matcher is orphaned as a result of this '
          'command, the command removes the orphaned path matcher instead '
          'of failing.'
      ),
  )

  group = parser.add_mutually_exclusive_group(required=True)
  group.add_argument(
      '--default-service',
      help=(
          'A backend service that will be used for requests that the path '
          'matcher cannot match. Exactly one of --default-service or '
          '--default-backend-bucket is required.'
      ),
  )
  group.add_argument(
      '--default-backend-bucket',
      help=(
          'A backend bucket that will be used for requests that the path '
          'matcher cannot match. Exactly one of --default-service or '
          '--default-backend-bucket is required.'
      ),
  )

  parser.add_argument(
      '--backend-service-path-rules',
      type=arg_parsers.ArgDict(min_length=1),
      default={},
      metavar='PATH=SERVICE',
      help='Rules for mapping request paths to services.',
  )
  parser.add_argument(
      '--backend-bucket-path-rules',
      type=arg_parsers.ArgDict(min_length=1),
      default={},
      metavar='PATH=BUCKET',
      help='Rules for mapping request paths to backend buckets.',
  )


def _GetGetRequest(client, url_map_ref):
  """Returns the request for the existing URL map resource."""
  return (
      client.apitools_client.urlMaps,
      'Get',
      client.messages.ComputeUrlMapsGetRequest(
          urlMap=url_map_ref.Name(), project=url_map_ref.project
      ),
  )


def _GetSetRequest(client, url_map_ref, replacement):
  return (
      client.apitools_client.urlMaps,
      'Update',
      client.messages.ComputeUrlMapsUpdateRequest(
          urlMap=url_map_ref.Name(),
          urlMapResource=replacement,
          project=url_map_ref.project,
      ),
  )


def _ModifyBase(client, args, existing):
  """Modifications to the URL map that are shared between release tracks.

  Args:
    client: The compute client.
    args: the argparse arguments that this command was invoked with.
    existing: the existing URL map message.

  Returns:
    A modified URL map message.
  """
  replacement = encoding.CopyProtoMessage(existing)

  if not args.new_hosts and not args.existing_host:
    new_hosts = ['*']
  else:
    new_hosts = args.new_hosts

  # If --new-hosts is given, we check to make sure none of those
  # hosts already exist and once the check succeeds, we create the
  # new host rule.
  if new_hosts:
    new_hosts = set(new_hosts)
    for host_rule in existing.hostRules:
      for host in host_rule.hosts:
        if host in new_hosts:
          raise compute_exceptions.ArgumentError(
              'Cannot create a new host rule with host [{0}] because the '
              'host is already part of a host rule that references the path '
              'matcher [{1}].'.format(host, host_rule.pathMatcher)
          )

    replacement.hostRules.append(
        client.messages.HostRule(
            hosts=sorted(new_hosts), pathMatcher=args.path_matcher_name
        )
    )

  # If --existing-host is given, we check to make sure that the
  # corresponding host rule will not render a patch matcher
  # orphan. If the check succeeds, we change the path matcher of the
  # host rule. If the check fails, we remove the path matcher if
  # --delete-orphaned-path-matcher is given otherwise we fail.
  else:
    target_host_rule = None
    for host_rule in existing.hostRules:
      for host in host_rule.hosts:
        if host == args.existing_host:
          target_host_rule = host_rule
          break
      if target_host_rule:
        break

    if not target_host_rule:
      raise compute_exceptions.ArgumentError(
          'No host rule with host [{0}] exists. Check your spelling or '
          'use [--new-hosts] to create a new host rule.'.format(
              args.existing_host
          )
      )

    path_matcher_orphaned = True
    for host_rule in replacement.hostRules:
      if host_rule == target_host_rule:
        host_rule.pathMatcher = args.path_matcher_name
        continue

      if host_rule.pathMatcher == target_host_rule.pathMatcher:
        path_matcher_orphaned = False
        break

    if path_matcher_orphaned:
      # A path matcher will be orphaned, so now we determine whether
      # we should delete the path matcher or report an error.
      if args.delete_orphaned_path_matcher:
        replacement.pathMatchers = [
            path_matcher
            for path_matcher in existing.pathMatchers
            if path_matcher.name != target_host_rule.pathMatcher
        ]
      else:
        raise compute_exceptions.ArgumentError(
            'This operation will orphan the path matcher [{0}]. To '
            'delete the orphan path matcher, rerun this command with '
            '[--delete-orphaned-path-matcher] or use [gcloud compute '
            'url-maps edit] to modify the URL map by hand.'.format(
                host_rule.pathMatcher
            )
        )

  return replacement


def _Modify(
    client,
    resources,
    args,
    url_map,
    url_map_ref,
    backend_service_arg,
    backend_bucket_arg,
):
  """Returns a modified URL map message."""
  replacement = _ModifyBase(client, args, url_map)

  # Creates PathRule objects from --path-rules, --backend-service-path-rules,
  # and --backend-bucket-path-rules.
  service_map = collections.defaultdict(set)
  bucket_map = collections.defaultdict(set)
  for path, service in args.path_rules.items():
    service_map[service].add(path)
  for path, service in args.backend_service_path_rules.items():
    service_map[service].add(path)
  for path, bucket in args.backend_bucket_path_rules.items():
    bucket_map[bucket].add(path)
  path_rules = []
  for service, paths in sorted(service_map.items()):
    path_rules.append(
        client.messages.PathRule(
            paths=sorted(paths),
            service=resources.Parse(
                service,
                params=_GetBackendParamsForUrlMap(url_map, url_map_ref),
                collection=_GetBackendServiceCollectionForUrlMap(url_map),
            ).SelfLink(),
        )
    )
  for bucket, paths in sorted(bucket_map.items()):
    path_rules.append(
        client.messages.PathRule(
            paths=sorted(paths),
            service=resources.Parse(
                bucket,
                params=_GetBackendParamsForUrlMap(url_map, url_map_ref),
                collection=_GetBackendBucketCollectionForUrlMap(url_map),
            ).SelfLink(),
        )
    )

  if args.default_service:
    default_backend_uri = url_maps_utils.ResolveUrlMapDefaultService(
        args, backend_service_arg, url_map_ref, resources
    ).SelfLink()
  else:
    default_backend_uri = url_maps_utils.ResolveUrlMapDefaultBackendBucket(
        args, backend_bucket_arg, url_map_ref, resources
    ).SelfLink()

  new_path_matcher = client.messages.PathMatcher(
      defaultService=default_backend_uri,
      description=args.description,
      name=args.path_matcher_name,
      pathRules=path_rules,
  )

  replacement.pathMatchers.append(new_path_matcher)
  return replacement


def _GetRegionalGetRequest(client, url_map_ref):
  """Returns the request to get an existing regional URL map resource."""
  return (
      client.apitools_client.regionUrlMaps,
      'Get',
      client.messages.ComputeRegionUrlMapsGetRequest(
          urlMap=url_map_ref.Name(),
          project=url_map_ref.project,
          region=url_map_ref.region,
      ),
  )


def _GetRegionalSetRequest(client, url_map_ref, replacement):
  """Returns the request to update an existing regional URL map resource."""
  return (
      client.apitools_client.regionUrlMaps,
      'Update',
      client.messages.ComputeRegionUrlMapsUpdateRequest(
          urlMap=url_map_ref.Name(),
          urlMapResource=replacement,
          project=url_map_ref.project,
          region=url_map_ref.region,
      ),
  )


def _GetBackendParamsForUrlMap(url_map, url_map_ref):
  params = {'project': properties.VALUES.core.project.GetOrFail}
  if hasattr(url_map, 'region') and url_map.region:
    params['region'] = url_map_ref.region

  return params


def _GetBackendServiceCollectionForUrlMap(url_map):
  if hasattr(url_map, 'region') and url_map.region:
    return 'compute.regionBackendServices'
  else:
    return 'compute.backendServices'


def _GetBackendBucketCollectionForUrlMap(url_map):
  if hasattr(url_map, 'region') and url_map.region:
    return 'compute.regionBackendBuckets'
  else:
    return 'compute.backendBuckets'


def _Run(args, holder, url_map_arg, backend_servie_arg, backend_bucket_arg):
  """Issues requests necessary to add path matcher to the Url Map."""
  client = holder.client

  url_map_ref = url_map_arg.ResolveAsResource(
      args, holder.resources, default_scope=compute_scope.ScopeEnum.GLOBAL
  )
  if url_maps_utils.IsRegionalUrlMapRef(url_map_ref):
    get_request = _GetRegionalGetRequest(client, url_map_ref)
  else:
    get_request = _GetGetRequest(client, url_map_ref)

  url_map = client.MakeRequests([get_request])[0]

  modified_url_map = _Modify(
      client,
      holder.resources,
      args,
      url_map,
      url_map_ref,
      backend_servie_arg,
      backend_bucket_arg,
  )

  if url_maps_utils.IsRegionalUrlMapRef(url_map_ref):
    set_request = _GetRegionalSetRequest(client, url_map_ref, modified_url_map)
  else:
    set_request = _GetSetRequest(client, url_map_ref, modified_url_map)

  return client.MakeRequests([set_request])


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
@base.UniverseCompatible
class AddPathMatcher(base.UpdateCommand):
  """Add a path matcher to a URL map."""

  detailed_help = _DetailedHelp()
  BACKEND_SERVICE_ARG = None
  BACKEND_BUCKET_ARG = None
  URL_MAP_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.BACKEND_BUCKET_ARG = (
        backend_bucket_flags.RegionSupportingBackendBucketArgumentForUrlMap(
            required=False
        )
    )
    cls.BACKEND_SERVICE_ARG = (
        backend_service_flags.BackendServiceArgumentForUrlMap()
    )
    cls.URL_MAP_ARG = flags.UrlMapArgument()
    cls.URL_MAP_ARG.AddArgument(parser)

    _Args(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    return _Run(
        args,
        holder,
        self.URL_MAP_ARG,
        self.BACKEND_SERVICE_ARG,
        self.BACKEND_BUCKET_ARG,
    )


@base.ReleaseTracks(base.ReleaseTrack.BETA)
@base.UniverseCompatible
class AddPathMatcherBeta(AddPathMatcher):
  """Add a path matcher to a URL map."""

  pass


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.UniverseCompatible
class AddPathMatcherAlpha(AddPathMatcherBeta):
  """Add a path matcher to a URL map."""

  pass



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to get capacity advice for Compute Engine resources."""

from apitools.base.py import encoding
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.compute import completers
from googlecloudsdk.command_lib.compute.advice import flags
from googlecloudsdk.core import properties


DETAILED_HELP = {
    "DESCRIPTION": """
      Get capacity advice for Compute Engine resources.

      This command helps you view future resource availability for a specific
      number of VM instances, machine type, provisioning model, and zone. After
      you confirm resource availability, you can specify those configurations
      when you create VM instances. This action improves the success rate of
      your VM instance creation request.
    """,
    "EXAMPLES": """
      To check the availability of 100 `n2-standard-32` Spot VMs in any single
      zone in the `us-central1` region, run the following command:

        $ {command} \
            --region="us-central1" \
            --provisioning-model="SPOT" \
            --size=100 \
            --instance-selection-machine-types="n2-standard-32" \
            --target-distribution-shape="any-single-zone"

      To check the availability of 50 Spot VMs, allowing either `e2-standard-8`
      or `e2-standard-16` machine types, distributed across `us-central1-a` and
      `us-central1-b`, run the following command:

        $ {command} \
            --region="us-central1" \
            --provisioning-model="SPOT" \
            --size=50 \
            --instance-selection="name=my-selection,machine-type=e2-standard-8,machine-type=e2-standard-16" \
            --target-distribution-shape="any" \
            --zones="us-central1-a,us-central1-b"

      To check the availability of 10 `ct5lp-hightpu-4t` Flex-start VMs that
      will run for 1 day in any single zone in the `us-central1` region, run the
      following command:

        $ {command} \
            --region="us-central1" \
            --provisioning-model="FLEX_START" \
            --max-run-duration=1d \
            --size=10 \
            --instance-selection-machine-types="ct5lp-hightpu-4t" \
            --target-distribution-shape="any-single-zone" \
            --accelerator-topology="2x2"
      """,
}


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class Capacity(base.Command):
  """Get capacity advice for Compute Engine resources."""

  detailed_help = DETAILED_HELP
  category = base.COMPUTE_CATEGORY

  @staticmethod
  def Args(parser):
    """Registers flags for this command."""
    flags.AddRegionFlag(parser)
    flags.AddProvisioningModelFlag(parser)
    flags.AddMaxRunDurationFlag(parser)
    flags.AddAcceleratorTopologyFlag(parser)
    parser.add_argument(
        "--size",
        type=int,
        required=True,
        help="The total number of VMs being requested in the capacity query.",
    )

    flags.AddInstanceFlexibilityPolicyArgs(parser)

    flags.AddTargetDistributionShapeFlag(parser)
    parser.add_argument(
        "--zones",
        type=arg_parsers.ArgList(),
        completer=completers.ZonesCompleter,
        metavar="ZONE",
        required=False,
        help=(
            "A comma-separated list of zones to query within the specified"
            " region, for example, `us-central1-a,us-central1-b`. If you omit"
            " this flag, then you view availability for your requested capacity"
            " across all zones in the region."
        ),
    )

  def Run(self, args):
    """Runs the capacity advice command."""
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    messages = client.messages
    flags.ValidateZonesAndRegionFlags(args, holder.resources)

    if args.instance_selection and len(args.instance_selection) > 1:
      raise exceptions.InvalidArgumentException(
          "--instance-selection",
          "Multiple instance selections are not supported. Please provide only"
          " one --instance-selection flag.",
      )
    if (
        args.instance_selection_machine_types
        and len(args.instance_selection_machine_types) > 1
    ):
      raise exceptions.InvalidArgumentException(
          "--instance-selection-machine-types",
          "Multiple instance selections are not supported. Please provide only"
          " one --instance-selection-machine-types flag.",
      )

    if args.instance_selection:
      for selection in args.instance_selection:
        if not selection.get("machine-type"):
          raise exceptions.InvalidArgumentException(
              "--instance-selection",
              "At least one 'machine-type' must be specified in each"
              " --instance-selection flag.",
          )

    project = properties.VALUES.core.project.GetOrFail()
    region = args.region
    if not region and args.zones:
      # All zones are in the same region, this is validated in
      # ValidateZonesAndRegionFlags.
      region = utils.ZoneNameToRegionName(args.zones[0])

    region = region or properties.VALUES.compute.region.Get()
    if not region:
      raise exceptions.RequiredArgumentException(
          "--region", "The [compute/region] property must be set.")

    # Instance Properties
    scheduling = messages.CapacityAdviceRequestInstancePropertiesScheduling(
        provisioningModel=messages.CapacityAdviceRequestInstancePropertiesScheduling.ProvisioningModelValueValuesEnum(
            args.provisioning_model
        )
    )

    if args.provisioning_model == "FLEX_START" and not args.IsSpecified(
        "max_run_duration"
    ):
      raise exceptions.RequiredArgumentException(
          "--max_run_duration",
          "The --max-run-duration flag is required when the provisioning model"
          " is FLEX_START.",
      )

    if args.IsSpecified("max_run_duration"):
      scheduling.maxRunDuration = f"{args.max_run_duration}s"

    instance_properties = messages.CapacityAdviceRequestInstanceProperties(
        scheduling=scheduling)
    if args.IsSpecified("accelerator_topology"):
      instance_properties.acceleratorTopology = args.accelerator_topology

    # Distribution Policy
    target_shape = None
    if args.IsSpecified("target_distribution_shape"):
      target_shape = (
          messages.CapacityAdviceRequestDistributionPolicy.TargetShapeValueValuesEnum(
              args.target_distribution_shape)
      )
    zone_configs = None
    if args.zones:
      zone_configs = []
      for zone in args.zones:
        zone_ref = holder.resources.Parse(
            zone,
            params={"project": project},
            collection="compute.zones")
        zone_configs.append(
            messages.CapacityAdviceRequestDistributionPolicyZoneConfiguration(
                zone=zone_ref.SelfLink()))
    distribution_policy = messages.CapacityAdviceRequestDistributionPolicy(
        targetShape=target_shape)
    if zone_configs:
      distribution_policy.zones = zone_configs
    selections_map = {}
    if args.instance_flexibility_policy:
      raw_selections = args.instance_flexibility_policy.get(
          "instanceSelections",
          args.instance_flexibility_policy.get("instance-selections", {})
      )
      items = []
      if isinstance(raw_selections, dict):
        items = list(raw_selections.items())
      elif isinstance(raw_selections, list):
        for sel in raw_selections:
          if not isinstance(sel, dict):
            raise exceptions.InvalidArgumentException(
                "--instance-flexibility-policy",
                "Each instance selection must be a dictionary.",
            )
          if "name" not in sel:
            raise exceptions.InvalidArgumentException(
                "--instance-flexibility-policy",
                "Missing instance selection name."
            )
          items.append((sel["name"], sel))

      for name, selection_info in items:
        machine_types = selection_info.get(
            "machineTypes",
            selection_info.get("machine-types", [])
        )
        selection = messages.CapacityAdviceRequestInstanceFlexibilityPolicyInstanceSelection(
            machineTypes=machine_types,
        )
        if "rank" in selection_info:
          selection.rank = int(selection_info["rank"])
        min_cpu = selection_info.get("minCpuPlatform") or selection_info.get(
            "min-cpu-platform"
        )
        if min_cpu:
          selection.minCpuPlatform = min_cpu
        if "disks" in selection_info:
          disks = []
          for disk_dict in selection_info["disks"]:
            disks.append(
                encoding.DictToMessage(disk_dict, messages.AttachedDisk)
            )
          selection.disks = disks
        selections_map[name] = selection
    elif args.instance_selection:
      for i, selection in enumerate(args.instance_selection):
        selection_name_list = selection.get("name")
        selection_name = (
            selection_name_list[0]
            if selection_name_list
            else "instance-selection-{}".format(i + 1)
        )
        if selection_name in selections_map:
          raise exceptions.InvalidArgumentException(
              "--instance-selection",
              "Duplicate instance selection name [{}] specified.".format(
                  selection_name
              ),
          )
        inst_sel = messages.CapacityAdviceRequestInstanceFlexibilityPolicyInstanceSelection(
            machineTypes=selection.get("machine-type"),
        )
        if "rank" in selection:
          inst_sel.rank = int(selection["rank"][0])
        if "min-cpu-platform" in selection:
          min_cpu_platform = selection["min-cpu-platform"]
          if isinstance(min_cpu_platform, list):
            inst_sel.minCpuPlatform = min_cpu_platform[0]
          else:
            inst_sel.minCpuPlatform = min_cpu_platform
        selections_map[selection_name] = inst_sel
    elif args.instance_selection_machine_types:
      for i, machine_types_list in enumerate(
          args.instance_selection_machine_types
      ):
        selection_name = "instance-selection-{}".format(i + 1)
        selections_map[selection_name] = (
            messages.CapacityAdviceRequestInstanceFlexibilityPolicyInstanceSelection(
                machineTypes=machine_types_list,
            )
        )

    additional_properties = []
    for key, value in selections_map.items():
      additional_properties.append(
          messages.CapacityAdviceRequestInstanceFlexibilityPolicy.InstanceSelectionsValue.AdditionalProperty(
              key=key, value=value)
      )

    instance_selections_value = (
        messages.CapacityAdviceRequestInstanceFlexibilityPolicy.InstanceSelectionsValue(
            additionalProperties=additional_properties
        )
    )
    instance_flexibility_policy = (
        messages.CapacityAdviceRequestInstanceFlexibilityPolicy(
            instanceSelections=instance_selections_value
        )
    )

    inner_request = messages.CapacityAdviceRequest(
        distributionPolicy=distribution_policy,
        instanceFlexibilityPolicy=instance_flexibility_policy,
        instanceProperties=instance_properties,
        size=args.size,
    )

    outer_request = messages.ComputeAdviceCapacityRequest(
        project=project,
        region=region,
        capacityAdviceRequest=inner_request,
    )

    return client.apitools_client.advice.Capacity(outer_request)



# Copyright 2017 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""ECDSA (ES256) verifier and signer that use the ``cryptography`` library.
"""

from google.auth.crypt.es import EsSigner
from google.auth.crypt.es import EsVerifier


class ES256Verifier(EsVerifier):
    """Verifies ECDSA cryptographic signatures using public keys.

    Args:
        public_key (cryptography.hazmat.primitives.asymmetric.ec.ECDSAPublicKey): The public key used to verify
            signatures.
    """

    pass


class ES256Signer(EsSigner):
    """Signs messages with an ECDSA private key.

    Args:
        private_key (
                cryptography.hazmat.primitives.asymmetric.ec.ECDSAPrivateKey):
            The private key to sign with.
        key_id (str): Optional key ID used to identify this private key. This
            can be useful to associate the private key with its associated
            public key or certificate.
    """

    pass



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities shared by cloud-shell commands."""


import argparse
import datetime
import time

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.command_lib.util.ssh import ssh
from googlecloudsdk.core import exceptions as core_exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core.credentials import store


DEFAULT_ENVIRONMENT_NAME = 'users/me/environments/default'

MIN_CREDS_EXPIRY = datetime.timedelta(minutes=30)
MIN_CREDS_EXPIRY_SECONDS = '{}s'.format(MIN_CREDS_EXPIRY.seconds)


class UnsupportedPlatform(core_exceptions.Error):
  """Raised when attempting to run a command on an unsupported platform."""


def ParseCommonArgs(parser):
  """Parses arguments common to all cloud-shell commands."""

  parser.add_argument(
      '--force-key-file-overwrite',
      help="""\
      If enabled gcloud will regenerate and overwrite the files associated
      with a broken SSH key without asking for confirmation in both
      interactive and non-interactive environment.

      If disabled gcloud will not attempt to regenerate the files associated
      with a broken SSH key and fail in both interactive and non-interactive
      environment.
      """,
      action='store_true')
  parser.add_argument(
      '--ssh-key-file',
      help="""\
      The path to the SSH key file. By default, this is
        *~/.ssh/google_compute_engine*.
      """,
      action='store_true')


def AddSshArgFlag(parser):
  parser.add_argument(
      'ssh_args',
      nargs=argparse.REMAINDER,
      help="""\
          Flags and positionals passed to the underlying ssh implementation.
          """,
      example="""\
        $ {command} -- -vvv
      """)


def PrepareEnvironment(args):
  """Ensures that the user's environment is ready to accept SSH connections."""

  # Load Cloud Shell API.
  client = apis.GetClientInstance('cloudshell', 'v1')
  messages = apis.GetMessagesModule('cloudshell', 'v1')
  operations_client = apis.GetClientInstance('cloudshell', 'v1')

  # Ensure we have a key pair on the local machine.
  ssh_env = ssh.Environment.Current()
  ssh_env.RequireSSH()
  keys = ssh.Keys.FromFilename(filename=args.ssh_key_file)
  keys.EnsureKeysExist(overwrite=args.force_key_file_overwrite)

  # Look up the Cloud Shell environment.
  environment = client.users_environments.Get(
      messages.CloudshellUsersEnvironmentsGetRequest(
          name=DEFAULT_ENVIRONMENT_NAME))

  # If the environment doesn't have the public key, push it.
  key = keys.GetPublicKey().ToEntry()
  has_key = False
  for candidate in environment.publicKeys:
    if key == candidate:
      has_key = True
      break
  add_public_key_error = None
  if not has_key:
    try:
      add_public_key_operation = client.users_environments.AddPublicKey(
          messages.CloudshellUsersEnvironmentsAddPublicKeyRequest(
              environment=DEFAULT_ENVIRONMENT_NAME,
              addPublicKeyRequest=messages.AddPublicKeyRequest(key=key),
          )
      )

      environment = waiter.WaitFor(
          EnvironmentPoller(
              client.users_environments, operations_client.operations
          ),
          add_public_key_operation,
          'Pushing your public key to Cloud Shell',
          sleep_ms=500,
          max_wait_ms=None,
      )
      # Wait for the public key to propagate. Unfortunately there is no
      # api we can check for this.
      log.Print('Waiting for your public key to propagate...')
      time.sleep(5)
      key_file = keys.key_file
    except apitools_exceptions.HttpError as e:
      if e.status_code == 410:
        log.Print('AddPublicKey is deprecated. Skipping public key push.')
        key_file = None
        add_public_key_error = e
      else:
        raise
  else:
    key_file = keys.key_file

  # If the environment isn't running, start it.
  start_operation = None
  if environment.state != messages.Environment.StateValueValuesEnum.RUNNING:
    log.Print('Starting your Cloud Shell machine...')

    access_token = None
    if args.IsKnownAndSpecified('authorize_session') and args.authorize_session:
      access_token = store.GetFreshAccessTokenIfEnabled(
          min_expiry_duration=MIN_CREDS_EXPIRY_SECONDS
      )

    start_operation = client.users_environments.Start(
        messages.CloudshellUsersEnvironmentsStartRequest(
            name=DEFAULT_ENVIRONMENT_NAME,
            startEnvironmentRequest=messages.StartEnvironmentRequest(
                accessToken=access_token
            ),
        )
    )

    environment = waiter.WaitFor(
        EnvironmentPoller(
            client.users_environments, operations_client.operations
        ),
        start_operation,
        'Waiting for your Cloud Shell machine to start',
        sleep_ms=500,
        max_wait_ms=None,
    )

  if not environment.sshHost:
    _RaiseExceptionDueToStartOperationError(start_operation)

  return ConnectionInfo(
      ssh_env=ssh_env,
      user=environment.sshUsername,
      host=environment.sshHost,
      port=environment.sshPort,
      key=key_file,
      web_host=environment.webHost,
      add_public_key_error=add_public_key_error,
  )


def _GetDetail(details, type_class):
  if details is None:
    return None

  for detail in details:
    for additional_property in detail.additionalProperties:
      if (
          additional_property.key == '@type'
          and additional_property.value.string_value == type_class
      ):
        return detail
  return None


def _GetAdditionalPropertyFromDetail(detail, key):
  if detail is None:
    return None

  for additional_property in detail.additionalProperties:
    if additional_property.key == key:
      return additional_property.value
  return None


def _RaiseExceptionDueToStartOperationError(operation):
  """Raises a core_exceptions.Error based on the start operation's error details.

  If the error is due to an unverified account, a detailed error including the
  verification URL (if it exists) will be raised. Otherwise, a generic error is
  raised.

  Args:
    operation: The operation object returned from the Cloud Shell API.
  """
  details = None
  if operation and operation.error:
    details = operation.error.details

  error_info = _GetDetail(details, 'type.googleapis.com/google.rpc.ErrorInfo')
  reason_property = _GetAdditionalPropertyFromDetail(error_info, 'reason')
  error_reason = reason_property.string_value if reason_property else None

  if error_reason != 'ACCOUNT_UNVERIFIED':
    raise core_exceptions.Error('The Cloud Shell machine did not start.')

  verify_url = None
  help_message = _GetDetail(details, 'type.googleapis.com/google.rpc.Help')

  # Attempt to retrieve the uplevel link from the operation help message.
  # If it doesn't exist, use the Cloud Shell homepage as a fallback.
  if help_message:
    links = _GetAdditionalPropertyFromDetail(help_message, 'links')
    if links:
      for entries in links.array_value.entries:
        if verify_url:
          break
        for link in entries.object_value.properties:
          if link.key == 'url':
            verify_url = link.value.string_value
            break
  if verify_url:
    raise core_exceptions.Error(
        'Your account is unverified. Please verify your account at'
        f' {verify_url} or at https://shell.cloud.google.com.'
    )
  else:
    raise core_exceptions.Error(
        'Your account is unverified. Please verify your account at'
        ' https://shell.cloud.google.com.'
    )


def AuthorizeEnvironment():
  """Pushes gcloud command-line tool credentials to the user's environment."""

  client = apis.GetClientInstance('cloudshell', 'v1')
  messages = apis.GetMessagesModule('cloudshell', 'v1')

  # Load creds and refresh them if they're about to expire.
  access_token = store.GetFreshAccessTokenIfEnabled(
      min_expiry_duration=MIN_CREDS_EXPIRY_SECONDS)
  if access_token:
    client.users_environments.Authorize(
        messages.CloudshellUsersEnvironmentsAuthorizeRequest(
            name=DEFAULT_ENVIRONMENT_NAME,
            authorizeEnvironmentRequest=messages.AuthorizeEnvironmentRequest(
                accessToken=access_token)))


def GenerateAccessToken(environment=DEFAULT_ENVIRONMENT_NAME):
  """Generates an access token for the user's environment."""
  client = apis.GetClientInstance('cloudshell', 'v1')
  messages = apis.GetMessagesModule('cloudshell', 'v1')
  req = messages.CloudshellUsersEnvironmentsGenerateAccessTokenRequest(
      environment=environment
  )
  res = client.users_environments.GenerateAccessToken(req)
  return res.accessToken


class ConnectionInfo(object):
  """Connection information for Cloud Shell."""

  def __init__(
      self,
      ssh_env,
      user,
      host,
      port,
      key,
      web_host=None,
      add_public_key_error=None,
  ):
    self.ssh_env = ssh_env
    self.user = user
    self.host = host
    self.port = port
    self.key = key
    self.web_host = web_host
    self.add_public_key_error = add_public_key_error


class EnvironmentPoller(waiter.OperationPoller):
  """Poller for environment operations."""

  def __init__(self, environments_service, operations_service):
    self.environments_service = environments_service
    self.operations_service = operations_service

  def IsDone(self, operation):
    return operation.done

  def Poll(self, operation):
    request_type = self.operations_service.GetRequestType('Get')
    return self.operations_service.Get(request_type(name=operation.name))

  def GetResult(self, operation):
    request_type = self.environments_service.GetRequestType('Get')
    return self.environments_service.Get(
        request_type(name=DEFAULT_ENVIRONMENT_NAME))
